package com.example.blum.myapplication;

import android.media.AudioAttributes;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.os.Handler;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;

public class MainActivity extends AppCompatActivity {

    int myIndex=0;  //controls what note is played

    Handler handler = new Handler();
    /*
    A Handler allows you to send and process Message and Runnable objects associated
    with a thread's MessageQueue. Each Handler instance is associated with a single
    thread and that thread's message queue. When you create a new Handler, it is bound
    to the thread / message queue of the thread that is creating it -- from that point
    on, it will deliver messages and runnables to that message queue and execute them
    as they come out of the message queue
    https://developer.android.com/reference/android/os/Handler.html
    */

    int sampleRate = 8000;
    int numSamples = 3200;  //.4 of a second * 8000
    double sample[] = new double[numSamples];
    byte generatedSnd[] = new byte[2 * numSamples];


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Button btnPlay = (Button) findViewById(R.id.btnPlay);

        btnPlay.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                final Thread thread = new Thread(new Runnable() {
                    public void run() {
                        double myFreq = 440*Math.pow(2,myIndex/12.0);
                        // 0 corresponds to 440 Hz A
                        // 1 corresponds to A#
                        // 2 corresponds to B
                        // etc.
                        genTone(myFreq);
                        handler.post(new Runnable() {

                            public void run() {
                                playSound();
                            }
                        });
                    }
                });
                thread.start();
                //myIndex++;  //changes frequency -- goes up a note
            }
        });

    }

    //*****************************************************************************************************
    void genTone(double freqOfTone){
        // fill out the array	   -- make a sine wave
        for (int i = 0; i < numSamples; ++i) {
            sample[i] = Math.sin(2 * Math.PI * i / (sampleRate/freqOfTone));
        }
        // convert to 16 bit pcm sound array
        // assumes the sample buffer is normalised.
        int idx = 0;
        for (final double dVal : sample) {
            // scale to maximum amplitude
            final short val = (short) ((dVal * 32767));
            // in 16 bit wav PCM, first byte is the low order byte
            generatedSnd[idx++] = (byte) (val & 0x00ff);
            generatedSnd[idx++] = (byte) ((val & 0xff00) >>> 8);

        }
    }
    //*******************************************************************************************************

    //*****************************************************************************************************
    void playSound(){

        final AudioTrack audioTrack = new AudioTrack(AudioManager.STREAM_MUSIC,
                sampleRate, AudioFormat.CHANNEL_OUT_MONO,
                AudioFormat.ENCODING_PCM_16BIT, generatedSnd.length,
                AudioTrack.MODE_STATIC);
        audioTrack.write(generatedSnd, 0, generatedSnd.length);
        audioTrack.play();
        /*
        I got caught in a kind of Catch 22.  It tells me the constructor for AudioTrack has
        been deprecated and that I should use Builder.  But when I attempt to use Builder
        it tells me that the minimum SDK I specified is too low for using the newer code.
        */

    }
    //******************************************************************************************************

}
